;; Generated package description from tempel.el  -*- mode: lisp-data; no-byte-compile: t -*-
(define-package "tempel" "1.10.0.20260121.214800" "Tempo templates/snippets with in-buffer field editing" '((emacs "29.1") (compat "30")) :commit "632a4478ed4ce5a77f3bf78a9551a003e30f6c1c" :authors '(("Daniel Mendler" . "mail@daniel-mendler.de")) :maintainer '("Daniel Mendler" . "mail@daniel-mendler.de") :keywords '("abbrev" "languages" "tools" "text") :url "https://github.com/minad/tempel")
